@@ -7,7 +7,7 @@
 #include "selinux_internal.h"
 #include "policy.h"
 
-int setfilecon_raw(const char *path, security_context_t context)
+int setfilecon_raw(const char *path, const security_context_t context)
 {
 	return setxattr(path, XATTR_NAME_SELINUX, context, strlen(context) + 1,
 			0);
@@ -15,10 +15,10 @@ int setfilecon_raw(const char *path, security_context_t context)
 
 hidden_def(setfilecon_raw)
 
-int setfilecon(const char *path, security_context_t context)
+int setfilecon(const char *path, const security_context_t context)
 {
 	int ret;
-	security_context_t rcontext = context;
+	security_context_t rcontext;
 
 	if (selinux_trans_to_raw_context(context, &rcontext))
 		return -1;
